#==================================================================
#  APL-UW High-Frequency Ocean Environmental Acoustic Models Handbook
#  Technical Report
#  APL-UW TR 9407
#  October 1994
#  Figure 2 p.148
#  Faro, Qui Set  9 21:00:36 WEST 2021
#  Written by Orlando Camargo Rodriguez
#==================================================================

# High-Frequency Bistatic Scattering Model for Elastic Seafloors
# Darrell R. Jackson 2000
# p.10:
# arho = rhob/rhow
# vp   = cp/cw
# vt   = cs/cw
# dp   = imag(kp)/real(kp)
# dt   = imag(ks)/real(ks)

# complex sound speed: 
# c_tilde = cp,s*( 1 - i*ap,s_tilde)/( 1+ (ap,s_tilde)**2 )
# ap,s_tilde = ap,s/( 40*pi*log10(e) )
# [ap] = [as] = dB/wavelength

from numpy import *
from matplotlib.pyplot import *
from apl_bottom_sigmalr import *
from apl_bottom_sigmav  import *
from apl_bottom_sigma   import *

cw      = 1500.0
freqkHz = 30; freq = freqkHz*1000
kw      = 2.0*pi*freq/cw
eta     = -2.0

sigma2 = 0.002

nthetas = 180
thetas  = linspace(1,90,nthetas)

sigmakr = zeros( nthetas )
sigmalr = zeros( nthetas )
sigmapr = zeros( nthetas )
sigmav  = zeros( nthetas ) # For hard bottoms it seems to be of little importance ...

arho = 2.5
vp   = 2.5
dp   = 0.01374
g2   = 2.6943
w2   = 0.0001

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmalr[i] = apl_bottom_sigmalr(q,g2,w2,kw,arho,vp,dp)
    sigmav[ i] = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

Sbroughrock = 10*log10( 4.0*sigmalr + sigmav ) # Yes, this is weird, but it approaches the table curve...

arho = 2.5
vp   = 2.3
dp   = 0.00174
g2   = 3.0
w2   = 0.00025

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmalr[i] = apl_bottom_sigmalr(q,g2,w2,kw,arho,vp,dp)
    sigmav[ i] = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

Sbrock = 10*log10( sigmalr + sigmav )

arho = 2.5	
vp   = 1.8
dp   = 0.01374

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmalr[i] = apl_bottom_sigmalr(q,g2,w2,kw,arho,vp,dp)
    sigmav[ i] = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

Scobble = 10*log10( sigmalr + sigmav )

arho = 2.492
vp   = 1.337
dp   = 0.01705
g2   = 3.0
w2   = 0.00018

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbsandygravel = 10*log10( sigmar + sigmav )

arho = 2.231
vp   = 1.2503
dp   = 0.01638
g2   = 3.25
w2   = 0.00022

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbcoarsesand = 10*log10( sigmar + sigmav )

arho = 1.845
vp   = 1.1782
dp   = 0.01624
g2   = 3.25
w2   = 0.0001406

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbmediumsand = 10*log10( sigmar + sigmav )

arho = 1.451	
vp   = 1.1072
dp   = 0.01602
g2   = 3.25
w2   = 0.000086

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbveryfinesand = 10*log10( sigmar + sigmav )

arho = 1.149
vp   = 0.9873
dp   = 0.00386
g2   = 3.25
w2   = 0.0000164

sigma2 = 0.001

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbsilta = 10*log10( sigmar + sigmav )

sigma2 = 0.0003

for i in range(nthetas):
    q = thetas[i]*pi/180.0
    sigmakr[i],sigmapr[i] = apl_bottom_sigma( q,q,pi,g2,w2,kw,arho,vp,dp)
    sigmav[ i]            = apl_bottom_sigmav(q,q,cw,freq,arho,vp,dp,sigma2)

sigmar = ( sigmakr**eta + sigmapr**eta )**(1.0/eta)

Sbsiltb = 10*log10( sigmar + sigmav )

figure(1)
plot(thetas,Sbroughrock   ,'b.' ,label='Rough Rock'    ,linewidth=2)
plot(thetas,Sbrock        ,'b--',label='Rock'          ,linewidth=2)
plot(thetas,Scobble       ,'b'  ,label='Cobble'        ,linewidth=2)
plot(thetas,Sbsandygravel ,'k--',label='Sandy Gravel'  ,linewidth=2)
plot(thetas,Sbcoarsesand  ,'k.' ,label='Coarse Sand'   ,linewidth=2)
plot(thetas,Sbmediumsand  ,'k-.',label='Medium Sand'   ,linewidth=2)
plot(thetas,Sbveryfinesand,'r'  ,label='Very Fine Sand',linewidth=2)
plot(thetas,Sbsilta       ,'r--',label=r'Silt $\sigma_2$ = 0.001' ,linewidth=2)
plot(thetas,Sbsiltb       ,'r-.',label=r'Silt $\sigma_2$ = 0.0003',linewidth=2)
xlabel('GRAZING ANGLE (deg)')
ylabel('SCATTERING STRENGTH (dB)')
ylim(-50,0)
legend(loc=4)
grid(True)

show()

print('done.')
