#==================================================================
#  JKPS
#  Figure 1.28(a) 
#  Monostatic backscattering strength at the bottom as a function of bottom type. 
#  (a) The NRL model at 3.0 kHz. 
#  Faro, Qui Set  9 21:16:41 WEST 2021
#  Written by Orlando Camargo Rodriguez
#==================================================================

from numpy import *
from matplotlib.pyplot import *
from nrl_ebottom_sigma import *

# Miller2015
# 
# Table 4.1 p.32
# [cp] = [cs] = m/s
# [ap] = [as] = dB/lambda
# Bottom parameters for multiple bottom types
#           rhob/rhow  cp/cw   cp   cs       ap  as
# Clay      1.5        1.00    1500 <100     0.2 1.0
# Silt      1.7        1.05    1575  80z^0.3 1.0 1.5
# Sand      1.9        1.1     1650 110z^0.3 0.8 2.5
# Gravel    2.0        1.2     1800 180z^0.3 0.6 1.5
# Moraine   2.1        1.3     1950  600     0.4 1.0
# Chalk     2.2        1.6     2400 1000     0.2 0.5
# Limestone 2.4        2.0     3000 1500     0.1 0.2
# Basalt    2.7        3.5     5250 2500     0.1 0.2
# 
# Table 4.4 p.41
# Mud       1.4 0.99-0.002i 0.1-0.0004i 3.3 0.000518
# Sand      2.0 1.2 -0.005i 0.3-0.077i  3.3 0.006957
# Limestone 2.4 2.7 -0.004i 1.5-0.003i  3.3 0.004
# Basalt    2.7 3.4 -0.006i 1.8-0.006i  3.3 0.01862

freq = 3000
h0 = 1.0
cw = 1500.0
g2 = 3.3

rho = array([1.4, 2.0, 2.4, 2.7])
cp  = array([0.99-1j*0.002,1.2-1j*0.005,2.7-1j*0.004,3.4-1j*0.006])*cw
cs  = array([0.1-1j*0.0004,0.3-1j*0.077,1.5-1j*0.003,1.8-1j*0.006])*cw
w2  = array([0.000518,0.006957,0.004,0.01862]) 

nthetas = 360
theta = linspace(1,90,nthetas)

sigma = zeros( (4,nthetas) ) + 1j*zeros( (4,nthetas) )

for i in range(nthetas):
    q = theta[i]*pi/180.0
    sigma[0,i] = nrl_ebottom_sigma(q,q,freq,cw,cp[0],cs[0],pi,rho[0],g2,w2[0],h0)
    sigma[1,i] = nrl_ebottom_sigma(q,q,freq,cw,cp[1],cs[1],pi,rho[1],g2,w2[1],h0)
    sigma[2,i] = nrl_ebottom_sigma(q,q,freq,cw,cp[2],cs[2],pi,rho[2],g2,w2[2],h0)
    sigma[3,i] = nrl_ebottom_sigma(q,q,freq,cw,cp[3],cs[3],pi,rho[3],g2,w2[3],h0)

SS = 10*log10( abs( sigma ) )

figure(1)
plot(theta,SS[0,],'k--',linewidth=2,label='Mud')
plot(theta,SS[1,],'k',linewidth=2,label='Sand')
plot(theta,SS[2,],'b-.',linewidth=2,label='Limestone')
plot(theta,SS[3,],'b',linewidth=2,label='Basalt')
ylim(-60,10)
xlabel('Grazing Angle (deg)' ,fontsize=18)
ylabel('Scattering Strength (dB)',fontsize=18)
title('NRL model - 3 kHz')
legend(loc=2)
grid(True)

show()

print('done.')

