#==================================================================
#  Cross section disk
#  Faro, Qui Set  9 21:17:44 WEST 2021
#  Written by Orlando Camargo Rodriguez
#==================================================================
# Miller2015
# 
# Table 4.1 p.32
# [cp] = [cs] = m/s
# [ap] = [as] = dB/lambda
# Bottom parameters for multiple bottom types
#           rhob/rhow  cp/cw   cp   cs       ap  as
# Clay      1.5        1.00    1500 <100     0.2 1.0
# Silt      1.7        1.05    1575  80z^0.3 1.0 1.5
# Sand      1.9        1.1     1650 110z^0.3 0.8 2.5
# Gravel    2.0        1.2     1800 180z^0.3 0.6 1.5
# Moraine   2.1        1.3     1950  600     0.4 1.0
# Chalk     2.2        1.6     2400 1000     0.2 0.5
# Limestone 2.4        2.0     3000 1500     0.1 0.2
# Basalt    2.7        3.5     5250 2500     0.1 0.2
# 
# Table 4.4 p.41
# Mud       1.4 0.99-0.002i 0.1-0.0004i 3.3 0.000518
# Sand      2.0 1.2 -0.005i 0.3-0.077i  3.3 0.006957
# Limestone 2.4 2.7 -0.004i 1.5-0.003i  3.3 0.004
# Basalt    2.7 3.4 -0.006i 1.8-0.006i  3.3 0.01862

from numpy import *
from scipy.io import *
from matplotlib.pyplot import *
from nrl_ebottom_sigma import *

freq = 10000
h0 = 1.0
cw = 1500.0
g2 = 3.3

rho = array([1.4, 2.0, 2.4, 2.7])
cp  = array([0.99-1j*0.002,1.2-1j*0.005,2.7-1j*0.004,3.4-1j*0.006])*cw
cs  = array([0.1-1j*0.0004,0.3-1j*0.077,1.5-1j*0.003,1.8-1j*0.006])*cw
w2  = array([0.000518,0.006957,0.004,0.01862])

nthetas = 90
theta = linspace(1.0,90.0,nthetas)
thetai = 45.0;
qi = thetai*pi/180.0

nphis = 181
phis  = linspace(0,360.0,nphis)

S     = zeros((nphis,nthetas))
sigma = zeros((nphis,nthetas))

for j in range(nphis):
    phi = phis[j]*pi/180.0
    for i in range(nthetas):
        q = theta[i]*pi/180.0
        sigma[j,i] = nrl_ebottom_sigma(qi,q,freq,cw,cp[1],cs[1],phi-pi,rho[1],g2,w2[1],h0)    

Sb = 10*log10( sigma )
[R,Bearings] = meshgrid(theta,phis)

X = R*cos( Bearings*pi/180.0 )
Y = R*sin( Bearings*pi/180.0 )

thetitle = r'Scattering Cross Section, $\theta_i = '+ str(thetai) + '^\circ$'

figure(1)
pcolor(X,Y,sigma,cmap='jet',vmin=0,vmax=0.35)
colorbar()
axis('off')
title( thetitle )
thetan = arange(15,90,15)
n = thetan.size
for i in range(n):
    s = str( thetan[i] )+r'$^\circ$'
    text(thetan[i],0,s,color='black')
n = 18
for i in range(9):
    s = str( i*20 )+r'$^\circ$'
    text(100*cos(i*2*pi/n),100*sin(i*2*pi/n),s,horizontalalignment='center',verticalalignment='center')

text(-100,0,r'$\pm$180$^\circ$',horizontalalignment='center',verticalalignment='center')

for i in range(9):
    s = str( -i*20 )+r'$^\circ$'
    text(100*cos(i*2*pi/n),100*sin(-i*2*pi/n),s,horizontalalignment='center',verticalalignment='center')

figure(2)
pcolor(X,Y,Sb,cmap='jet',vmin=-30,vmax=-10)
colorbar()
axis('off')
title( thetitle )
thetan = arange(15,90,15)
n = thetan.size
for i in range(n):
    s = str( thetan[i] )+r'$^\circ$'
    text(thetan[i],0,s,color='white')
n = 18
for i in range(9):
    s = str( i*20 )+r'$^\circ$'
    text(100*cos(i*2*pi/n),100*sin(i*2*pi/n),s,horizontalalignment='center',verticalalignment='center')

text(-100,0,r'$\pm$180$^\circ$',horizontalalignment='center',verticalalignment='center')

for i in range(9):
    s = str( -i*20 )+r'$^\circ$'
    text(100*cos(i*2*pi/n),100*sin(-i*2*pi/n),s,horizontalalignment='center',verticalalignment='center')

show()

print('done.')
