from numpy import *
from scipy.special import gamma,gammaln,kv
from scipy.signal  import *

def nrl_ebottom_sigma(theta1=None,theta2=None,freq=None,cw=None,cp=None,cs=None,sigma_bi=None,rho=None,g2=None,w2=None,h0=None):
    # NRL elastic bottom bistatic model
    #
    # SYNTAX: s = nrl_bottom_sigma(theta1,theta2,freq,cw,cp,cs,sigma_bi,rhob,gamma2,w2,h0)

    #*******************************************************************************
    # Faro, Qui Set  9 21:15:17 WEST 2021
    # Written by Orlando Camargo Rodriguez
    # Based on bssint.m (NRL Code 7144; January 2003)
    #*******************************************************************************

# References: 
# (1) Gragg, Wurmser, and Gauss, "Small Slope scattering from rough 
#     elastic ocean floors," JASA 110, 2878-2901 (2001).
# (2) R.C. Gauss, R.F. Gragg, R.W. Nero, D. Wurmser, and J.M. Fialkowski,
#     "Broadband Models for Predicting Bistatic Bottom, Surface, 
#     and Volume Scattering Strengths," NRL/FR/7100--02-10,042, 
#     Naval Research Laboratory, Washington DC, September 30, 2002.

# incident bistatic angle
# (set to 0 since assume tranversely isotropic wave spectra)
       rphi_in = 0.0

       rphi_out = sigma_bi

# empirical parameters
       ntilde = 40
       orders = array([6, 9])
       alpha_tilde = 10**(-1.7567*g2 + 4.4926) 

# secondary quantities
       nu = 0.5*( g2 - 2.0 )   
       omega = 2.0*pi*freq
       Kw = omega/cw
       hrms = sqrt( pi*w2/nu )/h0
       Kp = omega/cp
       Ks = omega/cs

# scattering directions
       rtheta_in = 0.5*pi + theta1
#      u_in = array([sin(rtheta_in)*cos(rphi_in), sin(rtheta_in)*sin(rphi_in), cos(rtheta_in)])
       u_in = array([sin(rtheta_in), 0.0, cos(rtheta_in)])
       rtheta_out = 0.5*pi - theta2
       u_out = array([sin(rtheta_out)*cos(rphi_out), sin(rtheta_out)*sin(rphi_out), cos(rtheta_out)])

# wave vectors
       k = u_in*Kw
       q = u_out*Kw   
       kdotq = k[0]*q[0] + k[1]*q[1]
       Q = k - q   
       absQhorz = sqrt( Q[0]*Q[0] + Q[1]*Q[1] )
       if absQhorz == 0.0:
          absQhorz = nan
# Dashen/Henyey/Wurmser (JASA, 1990) parameter
       alpha = ( gamma(1.0-nu)/gamma(1.0+nu)/4.0**nu )*( hrms*Q[2] )**2/( h0*absQhorz )**( 2.0*nu )

# bottom prefactor
       m = k
       mdm = m[0]*m[0] + m[1]*m[1]
       mpz = -sqrt( Kp**2 - mdm )
       msz = -sqrt( Ks**2 - mdm )
       mpr = m[2]/mpz
       msr = mdm/Ks**2
       a1  = rho*mpr* ( 1.0 - 4.0*msr* (msz/Ks)*( msz - mpz )/Ks )
       b1  = rho*mpr* ( 1.0 - 2.0*msr )
       xi1 = mpr* ( (msz - mpz)**2 - Kp**2 )/Ks**2
       
       m = -q
       mdm = m[0]*m[0] + m[1]*m[1]
       mpz = -sqrt( Kp**2 - mdm )
       msz = -sqrt( Ks**2 - mdm )
       mpr = m[2]/mpz
       msr = mdm/Ks**2
       a2	= rho*mpr*( 1.0 - 4.0*msr* (msz/Ks)* ( msz - mpz )/Ks )
       b2	= rho*mpr*( 1.0 - 2.0*msr )
       xi2	= mpr*( (msz - mpz)**2 - Kp**2 )/Ks**2

       beta1 = 4.0*(1.0-rho)*( k[2]*q[2] + kdotq*xi1*xi2)
       beta2 = 8.0*rho*(kdotq/Ks)**2*xi1*xi2
       beta3 = 4.0*kdotq*(a1-xi1)*(a2-xi2)
       beta4 = 4.0*Kw**2*a1*a2
       beta5 = 2.0*Ks**2*(a1-b1)*(a2-b2)/rho
       beta6 = 4.0*Kp**2*b1*b2/rho
       beta = ( beta1 + beta2 - beta3 + beta4 + beta5 - beta6 )/(1.0+a1)/(1.0+a2)

       prefactor = abs(beta/(Q[2]*absQhorz))**2/(8.0*pi)

# Small-slope asymptotic-series formulation (valid for "small" alpha). 
       if alpha < alpha_tilde:
          n = array(range(1,ntilde+1))
          np = n
          nup = n*nu
          X = -cos(pi*np)
          Y =  sin(pi*nup)*2.0/pi
          Z = exp( log(4.0)*nup + 2.0*gammaln(1.0+nup) - gammaln(1+n) + n*log(alpha) )
          integral = sum( X*Y*Z )
# Small-slope rational-function formulation (valid for non-small alpha):
       else:
#  local variables
          Num = orders[0]
          Den = orders[1]
          Tot = Num + Den
#     Set the approximation interval and interpolation points
          xmax = (-log(0.01))**(1/nu)   
          betaofnu = max(1.0,min(6.0,6.0-4.1*(nu-0.2)))  #  <-- Drumheller's original expression
          xaux = 1.0*array(range(1,Tot+1))/Tot
          x = xmax*xaux**betaofnu
          sx = size( x )
#     Fit with polynomial coefficients $nn = [n_Num, \cdots, n_1, 1]$, $dd = [d_Den, \cdots, d_1, 1]$.
          alpha0 = 1.0  #  <-- Drumheller's choice
          B = exp( -alpha0*x**nu )
          G = B - 1.0
          Maux = zeros((Den,sx))
          VV   = zeros((Den,sx))
          for ii in range(Den):
              Maux[ii,] = x
              VV[  ii,] = B
          QQ = cumprod( Maux, axis=0 );
          MVQ = concatenate((QQ[0:Num,],-VV*QQ),axis=0)
          temp = dot( G, linalg.inv(MVQ) )
          nn = temp[Num-1::-1]; nn = append(nn,[1.0]);
          dd = temp[Num+Den:Num-1:-1]; dd = append(dd,[1.0])
          res,pole,zilch = residue(nn,dd)
          zero = roots(nn)
#     Rotate the poles as necessary. 
          bad = ( real(pole) > 0 )&( imag(pole) == 0 )
          if any(bad): 
             shift = 0.5*pi 
             newpole = squeeze(array([pole[bad]*exp(1j*shift),pole[bad]*exp(-1j*shift)]))
             newzero = pole[bad]/(2*cos(shift)-1)
             pole = pole[~bad]
             pole = append(pole,newpole)
             zero = roots(nn)
             zero = append(zero,newzero)
             nn = poly(zero)/prod(-zero)
             dd = poly(pole)/prod(-pole)
             res,pole,zilch = residue(nn,dd)
#     Rotate the zeros as necessary. 
          bad = ( real(zero) > 0 )&( imag(zero) == 0 )
          if any(bad): 
             shift = 0.5*pi 
             newzero = squeeze(array([zero[bad]*exp(1j*shift),zero[bad]*exp(-1j*shift)]))
             newpole = zero[bad]/(2*cos(shift)-1)
             zero = zero[~bad]
             zero = append(zero,newzero)
             pole = roots(dd)
             pole = append(pole,newpole)
             nn = poly(zero)/prod(-zero)
             dd = poly(pole)/prod(-pole)   
             res,pole,zilch = residue(nn,dd)
#     Use the residues and poles in the analytic formulation to generate $I(\alpha)$ for all $\alpha$.
          scale = ones(size(res))*(alpha/alpha0)**(1.0/(2.0*nu))
          integral = real( sum( (res/(scale**2))* kv(0, sqrt(-pole)/scale ) ) )
 
       s = prefactor*integral
       
       if s > 1.0:
          s = 1.0

       return s
